package cn.iocoder.yudao.module.depository.service.warehouse.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.iocoder.yudao.framework.common.exception.ServiceException;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.depository.convert.warehouse.WarehouseConvert;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.DeptWarehouseDO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseDO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseLineDO;
import cn.iocoder.yudao.module.depository.dal.mysql.warehouse.DeptWarehouseMapper;
import cn.iocoder.yudao.module.depository.dal.mysql.warehouse.WarehouseLineMapper;
import cn.iocoder.yudao.module.depository.dal.mysql.warehouse.WarehouseMapper;
import cn.iocoder.yudao.module.depository.dto.ContactInfoDTO;
import cn.iocoder.yudao.module.depository.dto.LogisticsInfoDto;
import cn.iocoder.yudao.module.depository.service.warehouse.WarehouseService;
import cn.iocoder.yudao.module.depository.vo.dept.DeptWarehouseBaseVO;
import cn.iocoder.yudao.module.depository.vo.dept.DeptWarehouseRequestVo;
import cn.iocoder.yudao.module.depository.vo.warehouse.*;
import cn.iocoder.yudao.module.system.api.dict.DictDataApi;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import org.apache.commons.lang3.StringUtils;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.WAREHOUSE_NOT_EXISTS;

/**
 * 仓库 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class WarehouseServiceImpl implements WarehouseService {

    @Resource
    private WarehouseMapper warehouseMapper;
    @Resource
    private WarehouseLineMapper warehouseLineMapper;
    @Resource
    private DeptWarehouseMapper deptWarehouseMapper;
    @Resource
    private RedissonClient redissonClient;
    @Resource
    private DictDataApi dictDataApi;

    @Override
    public Long createWarehouse(WarehouseCreateReqVO createReqVO) {
        // 插入
        WarehouseDO warehouse = WarehouseConvert.INSTANCE.convert(createReqVO);
        warehouseMapper.insert(warehouse);
        // 返回
        return warehouse.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateWarehouse(WarehouseUpdateReqVO updateReqVO) {
        Long id = updateReqVO.getId();
        // 校验存在
        this.validateWarehouseExists(id);

        WarehouseDO dbWarehouseDO = getWarehouse(id);

        String oldTradeType = dbWarehouseDO.getTradeType();

        // 更新
        WarehouseDO updateObj = WarehouseConvert.INSTANCE.convert(updateReqVO);
        warehouseMapper.updateById(updateObj);

        String newTradeType = updateObj.getTradeType();

        //判断是否修改了
        if (!oldTradeType.equals(newTradeType)) {
            //修改了贸易属性。修改成进出口则不用管
            if (newTradeType.equals("3")) return;
            //老的是非进出口也不用管
            if (oldTradeType.equals("0")) return;
            if (oldTradeType.equals("3")) {
                //进出口改成了其他
                if (newTradeType.equals("1")) {
                    //改成了进口，删除出口的线路
                    warehouseLineMapper.delete(new LambdaQueryWrapperX<WarehouseLineDO>()
                            .eq(WarehouseLineDO::getStartWarehouseId, id)
                    );
                } else if (newTradeType.equals("2")) {
                    //改成了出口，删掉进口的线路
                    warehouseLineMapper.delete(new LambdaQueryWrapperX<WarehouseLineDO>()
                            .eq(WarehouseLineDO::getDestWarehouseId, id)
                    );
                } else {
                    //删除掉已经开通的线路
                    deleteWarehouseLine(id);
                }
            } else {
                //删除掉已经开通的线路
                deleteWarehouseLine(id);
            }
        }
    }

    private void deleteWarehouseLine(Long warehouseId) {
        warehouseLineMapper.delete(new LambdaQueryWrapperX<WarehouseLineDO>()
                .eq(WarehouseLineDO::getStartWarehouseId, warehouseId)
        );
        warehouseLineMapper.delete(new LambdaQueryWrapperX<WarehouseLineDO>()
                .eq(WarehouseLineDO::getDestWarehouseId, warehouseId)
        );
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteWarehouse(Long id) {
        // 校验存在
        this.validateWarehouseExists(id);
        // 删除
        warehouseMapper.deleteById(id);

        deleteWarehouseLine(id);
    }

    private void validateWarehouseExists(Long id) {
        if (warehouseMapper.selectById(id) == null) {
            throw exception(WAREHOUSE_NOT_EXISTS);
        }
    }

    @Override
    public WarehouseDO getWarehouse(Long id) {
        return warehouseMapper.selectById(id);
    }

    @Override
    public WarehouseDO getWarehouseByShi(Long shi) {
        return warehouseMapper.selectOne(new LambdaQueryWrapperX<WarehouseDO>().eq(WarehouseDO::getShi, shi).last("limit 1"));
    }

    @Override
    public List<WarehouseDO> getWarehouseList(Integer tradeType) {
        List<String> tradeTypeList = null;
        if (tradeType != null) {
            tradeTypeList = Arrays.asList("3", String.valueOf(tradeType));
        }
        return warehouseMapper.selectList(new LambdaQueryWrapperX<WarehouseDO>()
                .in(CollectionUtil.isNotEmpty(tradeTypeList), WarehouseDO::getTradeType, tradeTypeList)
                .eq(WarehouseDO::getStatus, 0)
        );
    }

    @Override
    public List<WarehouseDO> getByIdList(List<Long> idList) {
        return warehouseMapper.selectBatchIds(idList);
    }

    @Override
    public PageResult<WarehouseDO> getWarehousePage(WarehousePageReqVO pageReqVO) {
        return warehouseMapper.selectPage(pageReqVO);
    }

    @Override
    public List<WarehouseDO> getWarehouseList(WarehouseExportReqVO exportReqVO) {
        return warehouseMapper.selectList(exportReqVO);
    }

    @Override
    public List<WarehouseDO> getWarehouseRouterList(WarehouseLineRequestVO requestVO) {
        List<DictDataRespDTO> dictList = dictDataApi.getDictDatas("transport_type");
        List<String> existTypeList = dictList.stream()
                .map(DictDataRespDTO::getValue)
                .collect(Collectors.toList());

        //需要查找的
        String tradeType = requestVO.getTradeType();
        //当前仓库id
        Long id = requestVO.getId();
        WarehouseDO current = warehouseMapper.selectById(id);

        LambdaQueryWrapperX<WarehouseDO> query = new LambdaQueryWrapperX<>();
//        query.eq(WarehouseDO::getId, id);
        //3代表进出口仓库，
        List<String> tradeTypeList = new ArrayList<>(Collections.singletonList("3"));
        //加上筛选条件
        tradeTypeList.add(tradeType);
        query.in(WarehouseDO::getTradeType, tradeTypeList);
        //1.找到初步符合条件的所有仓库
        List<WarehouseDO> routeList = warehouseMapper.selectList(query);
        if (CollectionUtil.isEmpty(routeList)) return null;

        //查找出所有已经关联的线路
        List<WarehouseLineDO> lineList = warehouseLineMapper.selectList();

        WarehouseDO startWarehouse = null;

        Long startWarehouseId = 0L;
        Long destWarehouseId = 0L;

        List<WarehouseDO> matchDataList = new ArrayList<>();

        //2.筛选运输方式，只查找始发港才有的运输方式
//        String freight = current.getFreight();
//        String[] currentTransportArray = freight.split(",");
//        List<String> currentTransportList = new ArrayList<>(Arrays.asList(currentTransportArray));
        for (WarehouseDO warehouseDO : routeList) {
//            String otherFreight = warehouseDO.getFreight();
//            String[] otherTransportArray = otherFreight.split(",");
//            List<String> otherTransportList = new ArrayList<>(Arrays.asList(otherTransportArray));
            //两个集合求交集就是拥有相同的运输方式
//            currentTransportList.retainAll(otherTransportList);
            //遍历相同运输方式的集合，就是需要关联的路线

//            if(CollectionUtil.isEmpty(currentTransportList)) continue;

            if (tradeType.equals("1")) {  //关联进口-目的仓库，当前id出口-始发仓库
                startWarehouseId = id;
                startWarehouse = current;
                destWarehouseId = warehouseDO.getId();
            } else if (tradeType.equals("2")) { //关联出口-始发仓库，当前id是进口-目的仓库
                startWarehouseId = warehouseDO.getId();
                startWarehouse = warehouseDO;
                destWarehouseId = id;
            }

            String[] currentTransportArray = startWarehouse.getFreight().split(",");
            List<String> currentTransportList = new ArrayList<>(Arrays.asList(currentTransportArray));
            for (String transportType : currentTransportList) {
                //过滤掉禁用的运输方式
                if (!existTypeList.contains(transportType)) continue;

                WarehouseDO warehouseItem = ObjectUtil.clone(warehouseDO);
                //查看是否已经建立了关联关系
                //根据条件筛选路线
                Long finalStartWarehouseId = startWarehouseId;
                Long finalDestWarehouseId = destWarehouseId;
                Optional<WarehouseLineDO> optional = lineList.stream()
                        .filter(t -> t.getTransportType().equals(transportType))
                        .filter(t -> t.getStartWarehouseId().equals(finalStartWarehouseId))
                        .filter(t -> t.getDestWarehouseId().equals(finalDestWarehouseId))
                        .findFirst();
                if (optional.isPresent()) {
                    WarehouseLineDO lineDO = optional.get();
                    warehouseItem.setLineId(lineDO.getId());
                    warehouseItem.setOtherService(lineDO.getOtherService());
                    warehouseItem.setControlStatus(lineDO.getControlStatus());
                    warehouseItem.setCurrencyUnit(lineDO.getCurrencyUnit());
                    warehouseItem.setMinPrice(lineDO.getMinPrice());
                    warehouseItem.setMakeBillNode(lineDO.getMakeBillNode());
                    warehouseItem.setIsClientShow(lineDO.getIsClientShow());
                    warehouseItem.setTempTitleEn(lineDO.getTempTitleEn());
                    warehouseItem.setTempTitleZh(lineDO.getTempTitleZh());

                    //添加配置
                    warehouseItem.setLkRight(lineDO.getLkRight());
                    warehouseItem.setLkLeft(lineDO.getLkLeft());
                    warehouseItem.setTzName(lineDO.getTzName());

                    //end  添加配置

                    String recipientInfo = lineDO.getRecipientInfo();
                    if (recipientInfo != null) {
                        warehouseItem.setRecipientInfo(JSONObject.parseObject(recipientInfo, ContactInfoDTO.class));
                    }
                    String senderInfo = lineDO.getSenderInfo();
                    if (senderInfo != null) {
                        warehouseItem.setSenderInfo(JSONObject.parseObject(senderInfo, ContactInfoDTO.class));
                    }
                } else {
                    warehouseItem.setLineId(null);
                }
                warehouseItem.setTransportType(transportType);
                matchDataList.add(warehouseItem);
            }
        }

        return matchDataList;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long changeRouteStatus(WarehouseLineBindVO requestVO) {
        Long lineId = requestVO.getLineId();
        Long startWarehouseId = 0L;
        Long destWarehouseId = 0L;
        String tradeType = requestVO.getTradeType();
        Long id = requestVO.getId();
        if (tradeType.equals("1")) {  //关联进口-目的仓库，当前id出口-始发仓库
            startWarehouseId = id;
            destWarehouseId = requestVO.getWarehouseId();
        } else if (tradeType.equals("2")) { //关联出口-始发仓库，当前id是进口-目的仓库
            destWarehouseId = id;
            startWarehouseId = requestVO.getWarehouseId();
        }

        String transportType = requestVO.getTransportType();

        //始发仓、目的仓、运输方式决定线路，加锁，防止并发操作出现多条开通数据
        String lockKey = startWarehouseId + "-" + destWarehouseId + "-" + transportType;
        RLock rLock = redissonClient.getLock(lockKey);
        try {
            boolean success = rLock.tryLock(2, 2, TimeUnit.SECONDS);
            if (!success) throw new ServiceException(500, "操作忙，请稍后再试");
            //从线路里面找这个线路
            WarehouseLineDO warehouseLineDO = warehouseLineMapper.findOne(startWarehouseId, destWarehouseId, transportType);
            if (warehouseLineDO == null) {
                //新增
                WarehouseLineDO lineDO = new WarehouseLineDO();
                lineDO.setTransportType(requestVO.getTransportType());
                lineDO.setStartWarehouseId(startWarehouseId);
                lineDO.setDestWarehouseId(destWarehouseId);
                //默认支持控货
                lineDO.setControlStatus(0);
                //默认美元
                lineDO.setCurrencyUnit(1L);
                //默认排单
                lineDO.setMakeBillNode("2");
                //默认客户端显示
                lineDO.setIsClientShow(true);

                warehouseLineMapper.insert(lineDO);
                lineId = lineDO.getId();
            } else {
                if (warehouseLineDO.getControlStatus() == null) {
                    warehouseLineDO.setControlStatus(0); //控货
                }
                if (warehouseLineDO.getCurrencyUnit() == null || warehouseLineDO.getCurrencyUnit() == 0) {
                    warehouseLineDO.setCurrencyUnit(1L); //美元
                }
                if (StringUtils.isEmpty(warehouseLineDO.getMakeBillNode())) {
                    warehouseLineDO.setMakeBillNode("2"); //排单
                }
                if (warehouseLineDO.getIsClientShow() == null) {
                    warehouseLineDO.setIsClientShow(true); //默认客户端显示
                }
                warehouseLineMapper.updateById(warehouseLineDO);

                boolean deleted = warehouseLineDO.getDeleted();
                lineId = deleted ? warehouseLineDO.getId() : null;
                warehouseLineMapper.updateDeletedById(warehouseLineDO.getId(), !deleted);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (rLock.isLocked()) {
                rLock.unlock();
            }
        }
        return lineId;
    }

    @Override
    public List<WarehouseLineDO> openedRouterList(WarehouseLineSearchVO requestVO) {
        return warehouseLineMapper.openedRouterList(requestVO);
    }

    @Override
    public boolean quantityRequired(Long lineId) {
        WarehouseLineDO warehouseLineDO = warehouseLineMapper.selectById(lineId);
        String otherService = warehouseLineDO.getOtherService();
        return Boolean.FALSE.equals(warehouseLineDO.getDeleted())
                && StringUtils.isNotEmpty(otherService) && Arrays.asList(otherService.split(",")).contains("6");
    }

    @Override
    public List<WarehouseLineDO> selectBylineIdList(List<Long> lineIdList) {
        return warehouseLineMapper.selectBylineIdList(lineIdList);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Set<Long> deptBind(DeptWarehouseRequestVo requestVo) {
        List<Long> deptIdList = requestVo.getDeptIdList();
        Long warehouseId = requestVo.getWarehouseId();

        //首先删除这个仓库已绑定的所有部门
        deptWarehouseMapper.delete(new LambdaQueryWrapperX<DeptWarehouseDO>()
                .eq(DeptWarehouseDO::getWarehouseId, warehouseId)
        );

        List<DeptWarehouseDO> entityList = new ArrayList<>();
        for (Long deptId : deptIdList) {
            DeptWarehouseDO entity = new DeptWarehouseDO();
            entity.setDeptId(deptId);
            entity.setWarehouseId(warehouseId);
            entityList.add(entity);
        }
        deptWarehouseMapper.insertBatch(entityList);
        return entityList.stream()
                .map(DeptWarehouseDO::getDeptId)
                .collect(Collectors.toSet());
    }

    @Override
    public List<DeptWarehouseDO> deptList(DeptWarehouseBaseVO requestVo) {
        return deptWarehouseMapper.selectList(requestVo);
    }

    @Override
    public List<DeptWarehouseDO> deptList(List<Long> warehouseIdList) {
        return deptWarehouseMapper.selectList(new LambdaQueryWrapperX<DeptWarehouseDO>()
                .inIfPresent(DeptWarehouseDO::getWarehouseId, warehouseIdList)
        );
    }

    @Override
    public WarehouseRegionVO findWarehouseRegion(Long id) {
        return warehouseMapper.findWarehouseRegion(id);
    }

    @Override
    public List<WarehouseDO> findWarehouseListByCityIdAndTypes(Long cityId, List<Integer> types) {
        return warehouseMapper.findWarehouseListByCityIdAndTypes(cityId, types);
    }

    @Override
    public List<WarehouseDO> findByCountryIdList(List<Long> countryIdList) {
        return warehouseMapper.selectList(new LambdaQueryWrapperX<WarehouseDO>()
                .in(WarehouseDO::getGuojia, countryIdList)
        );
    }

    @Override
    public List<WarehouseDO> getAvailableWarehouseAdjustmentList(Long id) {
        WarehouseDO currentWarehouseDO = warehouseMapper.selectById(id);
        if (Objects.isNull(currentWarehouseDO)) {
            throw exception(WAREHOUSE_NOT_EXISTS);
        }
        return warehouseMapper.selectList(new LambdaQueryWrapper<WarehouseDO>()
                .eq(WarehouseDO::getGuojia, currentWarehouseDO.getGuojia())
                .notIn(WarehouseDO::getId, id)
                .orderByDesc(WarehouseDO::getId));
    }

    @Override
    public void serviceConfig(WarehouseLineServiceReqVO reqVO) {
        String senderInfo = null;
        if (reqVO.getSenderInfo() != null) {
            senderInfo = JSON.toJSONString(reqVO.getSenderInfo());
        }
        String recipientInfo = null;
        if (reqVO.getRecipientInfo() != null) {
            recipientInfo = JSON.toJSONString(reqVO.getRecipientInfo());
        }

        //添加配置参数项
        warehouseLineMapper.update(null,
                new LambdaUpdateWrapper<WarehouseLineDO>().eq(WarehouseLineDO::getId, reqVO.getLineId())
                        .set(WarehouseLineDO::getOtherService, reqVO.getOtherService())
                        .set(StringUtils.isNotEmpty(reqVO.getMakeBillNode()), WarehouseLineDO::getMakeBillNode, reqVO.getMakeBillNode())
                        .set(reqVO.getIsClientShow() != null, WarehouseLineDO::getIsClientShow, reqVO.getIsClientShow())
                        .set(reqVO.getCurrencyUnit() != null, WarehouseLineDO::getCurrencyUnit, reqVO.getCurrencyUnit())
                        .set(reqVO.getMinPrice() != null, WarehouseLineDO::getMinPrice, reqVO.getMinPrice())
                        .set(senderInfo != null, WarehouseLineDO::getSenderInfo, senderInfo)
                        .set(recipientInfo != null, WarehouseLineDO::getRecipientInfo, recipientInfo)
                        .set(StringUtils.isNotEmpty(reqVO.getTempTitleEn()), WarehouseLineDO::getTempTitleEn, reqVO.getTempTitleEn())
                        .set(StringUtils.isNotEmpty(reqVO.getTempTitleZh()), WarehouseLineDO::getTempTitleZh, reqVO.getTempTitleZh())
                        .set(reqVO.getControlStatus() != null, WarehouseLineDO::getControlStatus, reqVO.getControlStatus())
                        .set(StringUtils.isNotEmpty(reqVO.getLkLeft()), WarehouseLineDO::getLkLeft, reqVO.getLkLeft())
                        .set(StringUtils.isNotEmpty(reqVO.getLkRight()), WarehouseLineDO::getLkRight, reqVO.getLkRight())
                        .set(StringUtils.isNotEmpty(reqVO.getTzName()), WarehouseLineDO::getTzName, reqVO.getTzName())

        );
    }

    @Override
    public WarehouseLineDO getLineInfo(WarehouseLineReq warehouseLineReq) {
        return warehouseLineMapper.selectOne(new LambdaQueryWrapper<WarehouseLineDO>()
                .eq(WarehouseLineDO::getStartWarehouseId, warehouseLineReq.getStartWarehouseId())
                .eq(WarehouseLineDO::getDestWarehouseId, warehouseLineReq.getDestWarehouseId())
                .eq(WarehouseLineDO::getTransportType, warehouseLineReq.getTransportType())
        );
    }

    @Override
    public List<WarehouseLineDO> getLineInfoList(List<WarehouseLineReq> list) {
        List<WarehouseLineDO> lineList = new ArrayList<>();
        Map<String, WarehouseLineDO> lineMap = new HashMap<>();
        for (WarehouseLineReq req : list) {
            String key = req.getStartWarehouseId() + "_" + req.getDestWarehouseId() + "_" + req.getTransportType();
            WarehouseLineDO lineDO;
            if (lineMap.containsKey(key)) {
                lineDO = lineMap.get(key);
            } else {
                lineDO = getLineInfo(req);
                lineMap.put(key, lineDO);
            }
            lineList.add(lineDO);
        }
        return lineList;
    }

    @Override
    public LogisticsInfoDto getStartInfoAndDestInfoByLineId(Long lineId) {
        return warehouseLineMapper.getStartInfoAndDestInfoByLineId(lineId);
    }

    @Override
    public List<WarehouseTreeRegionVO> getWarehouseTreeRegionList(Integer tradeType) {
        List<WarehouseTreeRegionVO> list = warehouseMapper.getWarehouseTreeRegionList(tradeType);

        list.stream().forEach(s -> s.getGuojiaName().replaceAll("\t", ""));
        Map<String, List<WarehouseTreeRegionVO>> map = (HashMap<String, List<WarehouseTreeRegionVO>>) list.stream()
                .collect(Collectors.groupingBy(WarehouseTreeRegionVO::getGuojiaName));
        List<WarehouseTreeRegionVO> regionVOList = new ArrayList<>();
        for (Map.Entry<String, List<WarehouseTreeRegionVO>> m : map.entrySet()) {
            WarehouseTreeRegionVO v = new WarehouseTreeRegionVO();
            v.setGuojiaName(m.getKey());
            v.setChildren(m.getValue());
            v.setGuojia(m.getValue().get(0).getGuojia());
            v.setId(m.getValue().get(0).getGuojia());
            regionVOList.add(v);
        }

        return regionVOList;
    }

    @Override
    public List<WarehouseTreeRegionVO> getRegionList(Integer type, String regionId, String destCountryId, String objectiveId, String destWarehouseId) {
        if (regionId.equals("")) {
            return new ArrayList();
        }
        List<WarehouseTreeRegionVO> list =
                warehouseMapper.getRegionList(type, regionId, destCountryId, objectiveId, destWarehouseId);
        return list;
    }
}
